## @file kalkulačka_grafika.py
#  @brief This module provides the graphical interface for the calculator, communicates with matematicka_kniznica.py and outups the result.
#  @author Mário Baran, xbaranm00
#  @date 2024-04-24
#
#  This module creates a calculator interface allowing the user to perform
#  basic arithmetic operations. It supports keyboard and mouse input.

import tkinter as tk
import tkinter as tk1
from matematicka_kniznica import scitani, odcitani, nasobeni, deleni, mocnina, odmocnina, faktorial, ln, riesenie

# Nastavenie canvasu
root = tk.Tk()
# Meno canvasu
root.title("IVS-calculator")
cs=tk.Canvas(bg='antiquewhite', width=830, height=600)
cs.pack()

#display
display_area = cs.create_rectangle(3, 20, 810, 100, width=3, fill='white', outline='darkgray')
display_var = tk.StringVar()
display_var.set("0")  # Nastaví display na 0
# nastaví štýl a miesto zobrazenia čísel/ premenných najprv na začiatku postupne sa bude meniť
display_text = cs.create_text(10, 60, text=display_var.get(), fill='black', font='Arial 40', anchor='w')

##
#  @brief Resets the calculator display to 0
#  This function is triggered by the 'AC' button or 'c' key and clears the current input or result displayed on the canvas.

#vynuluje display
def clear_display():
    display_var.set("0")
    cs.itemconfigure(display_text, text=display_var.get())

#klavesnica znamienka + nápoveda-grafika
cs.create_rectangle(480,120,810,200, width=1,fill='darkgray',outline='darkgray') # AC
cs.create_text(643,160, text='AC', fill='lightgray',font='Arial 30')
cs.create_rectangle(650,210,810,290, width=1,fill='darkgray',outline='darkgray') # ÷
cs.create_text(730,252, text='÷', fill='lightgray',font='Arial 37')
cs.create_rectangle(650,300,810,380, width=1,fill='darkgray',outline='darkgray') # *
cs.create_text(730,353, text='*', fill='lightgray',font='Arial 40')
cs.create_rectangle(650,390,810,470, width=1,fill='darkgray',outline='darkgray') # -
cs.create_text(730,427, text='-', fill='lightgray',font='Arial 40')
cs.create_rectangle(650,480,810,560, width=1,fill='darkgray',outline='darkgray') # +
cs.create_text(730,523, text='+', fill='lightgray',font='Arial 40')
cs.create_rectangle(0,120,130,200, width=1,fill='darkgray',outline='darkgray') # nápoveda-?
cs.create_text(65,157, text='?', fill='lightgray',font='Arial 35')

#klavesnica cisla, desatinna ciarka a rovna sa-grafika
cs.create_rectangle(480,210,640,290, width=1,fill='darkgray',outline='darkgray') #9
cs.create_text(560,250, text='9', fill='lightgray',font='Arial 35')
cs.create_rectangle(310,210,470,290, width=1,fill='darkgray',outline='darkgray') #8
cs.create_text(390,250, text='8', fill='lightgray',font='Arial 35')
cs.create_rectangle(140,210,300,290, width=1,fill='darkgray',outline='darkgray') #7
cs.create_text(221,250, text='7', fill='lightgray',font='Arial 35')
cs.create_rectangle(480,300,640,380, width=1,fill='darkgray',outline='darkgray') #6
cs.create_text(560,340, text='6', fill='lightgray',font='Arial 35')
cs.create_rectangle(310,300,470,380, width=1,fill='darkgray',outline='darkgray') #5
cs.create_text(390,340, text='5', fill='lightgray',font='Arial 35')
cs.create_rectangle(140,300,300,380, width=1,fill='darkgray',outline='darkgray') #4
cs.create_text(221,340, text='4', fill='lightgray',font='Arial 35')
cs.create_rectangle(480,390,640,470, width=1,fill='darkgray',outline='darkgray') #3
cs.create_text(560,430, text='3', fill='lightgray',font='Arial 35')
cs.create_rectangle(310,390,470,470, width=1,fill='darkgray',outline='darkgray') #2
cs.create_text(390,430, text='2', fill='lightgray',font='Arial 35')
cs.create_rectangle(140,390,300,470, width=1,fill='darkgray',outline='darkgray') #1
cs.create_text(221,430, text='1', fill='lightgray',font='Arial 35')
cs.create_rectangle(480,480,640,560, width=1,fill='sky blue',outline='sky blue') #=
cs.create_text(560,520, text='=', fill='gray',font='Arial 35')
cs.create_rectangle(310,480,470,560, width=1,fill='darkgray',outline='darkgray') #.
cs.create_text(390,520, text='.', fill='lightgray',font='Arial 35')
cs.create_rectangle(140,480,300,560, width=1,fill='darkgray',outline='darkgray') #0
cs.create_text(221,520, text='0', fill='lightgray',font='Arial 35')

#umocnenie, obecná odmocnina, faktoriál-grafika
cs.create_rectangle(0,210,130,290, width=1,fill='darkgray',outline='darkgray') #x^n
cs.create_text(62,250, text='x', fill='lightgray',font='Arial 35')
cs.create_text(80,235, text='n', fill='lightgray',font='Arial 18')
cs.create_rectangle(0,300,130,380, width=1,fill='darkgray',outline='darkgray') #√
cs.create_text(45,343, text='√', fill='lightgray',font='Arial 35')
cs.create_text(70,343, text='x', fill='lightgray',font='Arial 30')
cs.create_text(40,323, text='n', fill='lightgray',font='Arial 18')
cs.create_rectangle(0,390,130,470, width=1,fill='darkgray',outline='darkgray') #x!
cs.create_text(58,430, text='x', fill='lightgray',font='Arial 35')
cs.create_text(77,430, text='!', fill='lightgray',font='Arial 35')
cs.create_rectangle(0,480,130,560, width=1,fill='darkgray',outline='darkgray') #ln
cs.create_text(60,520, text='ln', fill='lightgray',font='Arial 35')

## Function to handle mouse inputs 
#  @param event Mouse event that triggers this function
#  @details This function updates the display based on user input from the mouse click.
#           It handles digits, arithmetic operations, and evaluates expressions.

#reaguje na klikanie myši na rôzne políčka
def button_press(event):
    x, y = event.x, event.y
    ### Klik na nápovedu - '?' ###
    if 0 < x < 130 and 120 < y < 200:
        show_help()
    ### Klik na '9'###
    if 480 < x < 640 and 210 < y < 290:
        operation = '9'
    ### Klik na '8'###
    elif  310 < x < 470 and 210 < y < 290:
        operation = '8'
    ### Klik na '7'###
    elif  140 < x < 300 and 210 < y < 290:
        operation = '7'
    ### Klik na '6'###
    elif 480 < x < 640 and 300 < y < 380:
        operation = '6'
    ### Klik na '5'###
    elif 310 < x < 470 and 300 < y < 380:
        operation = '5'
    ### Klik na '4'###
    elif 140 < x < 300 and 300 < y < 380:
        operation = '4'
    ### Klik na '3'###
    elif 480 < x < 640 and 390 < y < 470:
        operation = '3'
    ### Klik na '2'###
    elif 310 < x < 470 and 390 < y < 470:
        operation = '2'
    ### Klik na '1'###
    elif 140 < x < 300 and 390 < y < 470:
        operation = '1'
    ### Klik na '0'###
    elif 140 < x < 300 and 480 < y < 560:
        operation = '0'
    ### Klik na '.'-desatinná čiarka###
    elif 310 < x < 470 and 480 < y < 560:
        operation = '.'
    ### Klik na '='###
    elif 480 < x < 640 and 480 < y < 560:
        evaluate_expression()
    ### Klik na '+'###
    elif 650 < x < 810 and 480 < y < 560:
        operation = '+'
    ### Klik na '-'###
    elif 650 < x < 810 and 390 < y < 470:
        operation = '-'
    ### Klik na '*'###
    elif 650 < x < 810 and 300 < y < 380:
        operation = '*'
    ### Klik na '÷'###
    elif 650 < x < 810 and 210 < y < 290:
        operation = '/'
    ### Klik na 'AC'###
    elif 480 < x < 810 and 120 < y < 200:
        clear_display()  
        #nastaví display na nulu
    ### Klik na 'x^n'###
    elif  0 < x < 130 and 210 < y < 290:
        operation = '^'
    ### Klik na '√'###
    elif  0 < x < 130 and 300 < y < 380:
        operation = 'r'
    ### Klik na '!'###
    elif  0 < x < 130 and 390 < y < 470:
        operation = '!'
    ### Klik na 'ln'###
    elif  0 < x < 130 and 480 < y < 560:
        #doplňujúca funkcia
        operation ='l'

#táto časť zaisťuje korektné zobrazovanie na displeji pre klávesové vstupy
    max_length = 27  # Maximum čo náš display vie zobraziť
    current_display = display_var.get()  # Získame aktuálny string displaja

    if current_display == "0" or current_display == "Error":
        # Ak je nové číslo 0 ignorujeme ho
        new_display = operation
    else:
        # Pridanie nového znaku
        new_display = current_display + operation

    #kontrola či sme nepresiahli limit
    if len(new_display) > max_length:
        # ponecháva len najnovšie hodnoty/znaky
        new_display = new_display[-max_length:]

    display_var.set(new_display)  # Update premennej na novú hodnotu
    cs.itemconfigure(display_text, text=new_display)  # Update canvasu

##
# @brief Function to evaluate expression
#

def evaluate_expression():
    expression = display_var.get()
    try:
        from matematicka_kniznica import riesenie  # riešenie pracuje so stringom
        result = riesenie(expression)  # vráti výsledok
        display_var.set(result)
    except Exception as e:
        display_var.set("Error")
    cs.itemconfigure(display_text, text=display_var.get())

##
#  @brief Function to open window with help for a user
#

#zobrazí okno s nápovedou
def show_help():
    # vytvorí nové okno s novým nadpisom
    help_window = tk1.Toplevel()
    help_window.title("Nápoveda")

    # Nastavuje veľkosť okna s nápovedou
    help_window.geometry("400x560+500+300")  # width x height + x_offset + y_offset
    help_canvas = tk1.Canvas(help_window, width=400, height=650, bg='white')
    help_canvas.pack(pady=10, padx=10)

    # Text nápovedy
    help_text = """
    IVS calculator - nápoveda:
    -------------------------------------------
    Základné operácie:
    + : Sčítanie    | Kláves: '+'
    - : Odčítanie   | Kláves: '-'
    * : Násobenie   | Kláves: '*'
    ÷ : Delenie     | Kláves: '/'

    Pokročilé funkcie:
    ^ : Mocniny     |
    √ : Odmocniny   | Kláves: 'r' (root)
    (na displeji zobrazená ako 'r' )
    ! : Faktoriál   | Kláves: '!'
    ln: pirodený logaritmus  | Kláves: 'l'
    (na displeji zobrazný ako 'l')

    Interakcia:
    AC : Vymazanie všetkých vstupov | Kláves: 'c'
    = : Vyhodnotí výraz  | Kláves: '='/<Return>

    Numerické znaky:
    Všetky majú klávesovú skratku prislúchajúcu
    ich hodnote
    Napr.: číslo 9: Kláves '9'
    -------------------------------------------
    Maximálna dĺžka vstupu je 27 znakov.
    V prípade prekročenia limitu sa zobrazia len
    posledné znaky, počíta sa avšak so všetkými.
    Kalkulačka zaokrúhluje na 8 desatinných miest.

    """
    help_canvas.create_text(10, 10, anchor='nw', text=help_text, font='Arial 11', fill='black')

## Function to handle mouse inputs 
#  @param event The keyboard event triggering this function
#  @details This function updates the display based on user input from the keyboard.
#           It handles digits, arithmetic operations, and evaluates expressions.

#vymazanie/ updatnutie displayu z klávesnice
def keyboard_input(event):
    max_length = 27
    # Nájde stlačenú klávesu
    key_char = event.char
    
    if key_char.isdigit() or key_char in ['+', '-', '*', '/', '.', 'l', 'r', '!']: #všetky funkcie pre ktoré chceme klávesový shortcut
        current_display = display_var.get()
        
        if current_display == "0":
            #Ak znova stlačíme "0" display sa neposunie, len sa nula nahradí
            display_var.set(key_char if key_char != '0' else "0")
        else:
            # Pridanie nového charakteru na dispaly
            new_display = current_display + key_char

            if len(new_display) > max_length:
                new_display = new_display[-max_length:]
        
            display_var.set(new_display)

    elif key_char == '=':
        # vyhodnotí výsledok príkladu
        evaluate_expression()
    
    elif key_char.lower() == 'c':
        # c vynuluje display
        clear_display()

    # vypíše nový text na canvas
    cs.itemconfigure(display_text, text=display_var.get())

    # Nabindovanie Entru na vyhodnotenie funkcie
    root.bind("<Return>", lambda e: evaluate_expression()) #klasický enter
    root.bind("<KP_Enter>", lambda e: evaluate_expression()) #enter numerickej klávesnice

# Bind pre myš
cs.bind("<Button-1>", button_press)
#Bind pre klávesnicu
root.bind("<Key>", keyboard_input)

root.mainloop()
