## @file matematicka_kniznica.py
#  @brief This mathematical library consists of basic mathematical operations such as sum, difference, multiplication or division, advanced mathematical operations such as power, root, natural logarithm. 
#  @author Adam Jakubovič, xjakuba00
#  @date 2024-04-24
#
# This mathematical library consists of basic mathematical operations such as sum, difference, multiplication or division, advanced mathematical operations such as power, root, natural logarithm. 


##
# @brief Sum of two numbers
#
# @param a First number to be added
# @param b Second number to be added
#
# @return Sum of two numbers a and b

@staticmethod
def scitani(a, b):
    return round(a + b, 8)

##
# @brief Difference of two numbers
#
# @param a First number, minor
# @param b Second number, minority
#
# @return Difference of two numbers a and b

@staticmethod
def odcitani(a, b):
    return round(a - b, 8)

##
# @brief Multiplication of two numbers
#
# @param a First number to be multiplied
# @param b Second number to be multiplied
#
# @return Multiplication of two numbers a and b

@staticmethod
def nasobeni(a, b):
    return round(a * b, 8)

##
# @brief Division of two numbers
#
# @param a First number, divident
# @param b Second number, divisor
#
# @return Division of two numbers a and b
# @exception Nelze dělit nulou! In case the second number is zero, function will throw error Nelze dělit nulou!

@staticmethod
def deleni(a, b):
    if b != 0:
        return round(a / b, 8)
    else:
        raise ValueError("Nelze dělit nulou!")

##
# @brief Power of two numbers
#
# @param a First number, base
# @param b Second number, exponent
#
# @return Power of two numbers a and b
# @exception Neplatny vstup pre mocninu, pre odmocnovanie pouzi odmocninu! In case the exponent is greater than zero and lesser than one, function will throw error Neplatny vstup pre mocninu, pre odmocnovanie pouzi odmocninu! because if you want to get root of it you should use root function, we will not allow you to use power function as root

@staticmethod
def mocnina(a, b):
    if (b > 0) and (b < 1):
        raise ValueError("Neplatny vstup pre mocninu, pre odmocnovanie pouzi odmocninu!")
    return round(a ** b, 8)

##
# @brief Root of two numbers
#
# @param a First number, radicant
# @param b Second number, index
#
# @return Root of two numbers a and b
# @exception Neplatny vstup pre odmocninu In case the radicant is lesser than zero or index is zero, function will throw error Neplatny vstup pre odmocninu

@staticmethod
def odmocnina(a, b):
    if (a < 0) or (b==0):
        raise ValueError("Neplatny vstup pre odmocninu")
    else:
        return round(a ** (1/b), 8)
##
# @brief Factorial of one number
#
# @param n First number, argument
#
# @return Factorial of one number
# @exception Factorial is not defined for negative numbers In case the argument is lesser than zero, function will throw error Factorial is not defined for negative numbers

@staticmethod
def faktorial(n):
    if n < 0:
        raise ValueError("Factorial is not defined for negative numbers")
    if n == 0:
        return 1
    result = 1
    for i in range(1, n + 1):
        result *= i
    return result

##
# @brief Natural logarithm of one number
#
# @param x First number, argument
#
# @return Natural logarithm of one number
# @exception Chyba In case the argument is lesser or equal to zero, function will throw error Chyba

@staticmethod
def ln(x):
    if x <= 0:
        raise ValueError("Chyba")

    n = 100000000.0
    return round(n * ((x ** (1/n)) - 1), 8)

##
# @brief Solve mathematical expression
#
# @param zadanie Mathematical expression in string form
#
# @return Resulf of the Mathematical expression
# @exception Neplatny vstup In case there are more than 2 operants like "-+" next to each other in zadanie, function will throw error Neplatny vstup
# @exception Nespravny pocet operantov In case there is invalid number of operants next to each other in zadanie, function will throw error Nespravny pocet operantov


@staticmethod
def riesenie(zadanie):
    zadanie = zadanie.replace(" ","")
    rozdelenie = []
    posledne_skoncene=-1
    operatory = ["+", "-", "*", "/", "^", "r", "!", "l"]
    prioritna_tabulka = [["l"], ["!"], ["^", "r"], ["/", "*"], ["+", "-"]] # tabulka priorit operantov
    cisla = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]
    znamienko = False # flag ktory pozera ci bol pouzity operant a caka na zmanieko
    pocitadlo_znamienok = 0 # pocitadlo_znamienok aby ich nebolo vela za sebou
    prve = True # flak ktory pozerac ci sa operant nachadza prvy a ak hej tak tak napise "-5" nie "-" "5"
    # zadanie zoberieme a potriedime to zvlast na list a prvky v nom
    # nesmieme zabudnut ked sa da za operant - alebo + tak to neni chyba ale je to znamienko
    # toto znamienko moze byt len raz ziadne ++2 nebude!!! zakazujem
    oprava_po_faktoriale = False # po prevedeni faktorialu sa to postupnost pokazi tak ju treba opravit

    # rozdelenie zadania do listu
    # zakladna idea je ze pozerame ci nenajdeme operant
    # po najdeni oprantu pridame do listu vsetko co je pred nim a nasledne operant
    # zapiseme posledne miesto pokial sme to zapisali a ideme dalej
    # najdeme operant a zase vypiseme od posledneho miesta po operant a potom operant
    for i in range(len(zadanie)):
        if i > 0:
            prve = False
        momentalny_znak = zadanie[i]

        # kontrolujeme ci neni za sebou prilis vela znamienok
        if pocitadlo_znamienok > 2:
            raise ValueError("Neplatny vstup")

        # ak narazime na cislo tak pocitadlo_znamienok nastavime na nulu a flag znamienka tiez
        if momentalny_znak in cisla:
            znamienko = False
            pocitadlo_znamienok = 0

        # ked narazime na operant tak:
        elif momentalny_znak in operatory:

            # musel som to osetrit pre ln lebo to vypisovalo zle ked bolo ln na zaciatku
            if prve and momentalny_znak != "l":
                prve = False

            # pocitam znamienka
            elif znamienko and momentalny_znak in prioritna_tabulka[4]:
                pocitadlo_znamienok += 1
            
            # ak je znamienko ine ako +- tak to hodi error nesmie byt za sebou */
            elif znamienko and momentalny_znak not in prioritna_tabulka[4] and momentalny_znak != "l":
                raise ValueError("Nespravny pocet operantov")

            # opravujeme po faktorialu takym sposobom ze nenastavujeme kde sme na posledy skoncili
            elif oprava_po_faktoriale:
                posledne_skoncene = i
                rozdelenie.append(momentalny_znak)
                oprava_po_faktoriale = False
                znamienko = True
                pocitadlo_znamienok += 1

            # specialne pisanie do listu po faktoriale lebo musime nastavit flag
            elif momentalny_znak == "!":
                rozdelenie.append(zadanie[posledne_skoncene+1:i])
                posledne_skoncene = i
                rozdelenie.append(momentalny_znak)
                oprava_po_faktoriale = True
                pocitadlo_znamienok += 1

            # pri ln sa to pise inak tiez lebo tam nas nezaujima co je pred operantom
            elif momentalny_znak == "l":
                rozdelenie.append(momentalny_znak)
                posledne_skoncene = i
                znamienko = True

            # povodna funkcia rozdelenia kde najdeme znamienko a pridame do rozdeleneho listu
            # cely string od poslendeho znamienka az po terajsie a potom aj terajsie
            else:
                rozdelenie.append(zadanie[posledne_skoncene+1:i])
                posledne_skoncene = i
                rozdelenie.append(momentalny_znak)
                znamienko = True
                pocitadlo_znamienok += 1

    # poslene cislo po operante sa nezapise tak ho musime zapisat tatko
    rozdelenie.append(zadanie[posledne_skoncene+1:])

    
    # tuto sa nachadza samotne riesenie zadania prejdeme rozdelenie niekolkokrat vzdy od najviac priorizovaneho operantu
    # postupne sa list rozdelenie skracuje a zjednodusuju az sa to vyriesi
    for operant in prioritna_tabulka:
        posuvanie = 0 # 
        while posuvanie < len(rozdelenie):
            policko = rozdelenie[posuvanie]
            if policko in operant:
                if policko == "+":
                    vysledok = [str(scitani(float(rozdelenie[posuvanie-1]), float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "-":
                    vysledok = [str(odcitani(float(rozdelenie[posuvanie-1]), float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "*":
                    vysledok = [str(nasobeni(float(rozdelenie[posuvanie-1]), float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "/":
                    vysledok = [str(deleni(float(rozdelenie[posuvanie-1]), float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "^":
                    vysledok = [str(mocnina(float(rozdelenie[posuvanie-1]), float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "r":
                    vysledok = [str(odmocnina(float(rozdelenie[posuvanie+1]), float(rozdelenie[posuvanie-1])))]
                    rozdelenie[posuvanie-1:posuvanie+2] = vysledok
                    posuvanie -= 1

                elif policko == "!":
                    vysledok = [str(faktorial(int(rozdelenie[posuvanie-1])))]
                    rozdelenie[posuvanie-1:posuvanie+1] = vysledok
                    posuvanie -= 1

                elif policko == "l":
                    vysledok = [str(ln(float(rozdelenie[posuvanie+1])))]
                    rozdelenie[posuvanie:posuvanie+2] = vysledok
                    posuvanie -= 1
            posuvanie += 1
    return(round(float(rozdelenie[0]), 8))

